import { ComputedRef, Ref, ComponentPublicInstance, ComponentOptionsBase } from 'vue';

interface ValidationResult {
    errors: string[];
}
interface FormController {
    register(field: any): void;
    values: ComputedRef<Record<string, any>>;
    fields: ComputedRef<Record<string, any>>;
    schema?: Record<string, GenericValidateFunction | string | Record<string, any>>;
    validateSchema?: (shouldMutate?: boolean) => Promise<Record<string, ValidationResult>>;
}
declare type MaybeReactive<T> = Ref<T> | ComputedRef<T> | T;
declare type SubmitEvent = Event & {
    target: HTMLFormElement;
};
declare type SubmissionHandler = (values: Record<string, any>, evt?: SubmitEvent) => any;
declare type GenericValidateFunction = (value: any) => boolean | string | Promise<boolean | string>;

interface ValidationOptions {
    name?: string;
    values?: Record<string, any>;
    bails?: boolean;
    skipIfEmpty?: boolean;
    isInitial?: boolean;
}
/**
 * Validates a value against the rules.
 */
declare function validate(value: any, rules: string | Record<string, any> | GenericValidateFunction, options?: ValidationOptions): Promise<ValidationResult>;

interface FieldContext {
    field: string;
    value: any;
    form: Record<string, any>;
    rule: {
        name: string;
        params?: Record<string, any> | any[];
    };
}
declare type ValidationRuleFunction = (value: any, params: any[] | Record<string, any>, ctx: FieldContext) => boolean | string | Promise<boolean | string>;
declare type ValidationMessageGenerator = (ctx: FieldContext) => string;

/**
 * Adds a custom validator to the list of validation rules.
 */
declare function defineRule(id: string, validator: ValidationRuleFunction): void;

interface VeeValidateConfig {
    bails: boolean;
    generateMessage: ValidationMessageGenerator;
    skipOptional: boolean;
}
declare const configure: (cfg: Partial<VeeValidateConfig>) => void;

declare const Field: (new () => ComponentPublicInstance<{
    name: string;
} & {
    disabled?: boolean;
    bails?: boolean;
    immediate?: boolean;
    as?: any;
    rules?: any;
}, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, import("vue").VNodeProps & import("vue").AllowedComponentProps & import("vue").ComponentCustomProps, import("vue").ComponentOptionsBase<{
    name: string;
} & {
    disabled?: boolean;
    bails?: boolean;
    immediate?: boolean;
    as?: any;
    rules?: any;
}, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string>>) & ComponentOptionsBase<Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string> & {
    props: {
        as: {
            type: (ObjectConstructor | StringConstructor)[];
            default: any;
        };
        name: {
            type: StringConstructor;
            required: true;
        };
        rules: {
            type: (ObjectConstructor | FunctionConstructor | StringConstructor)[];
            default: any;
        };
        immediate: {
            type: BooleanConstructor;
            default: boolean;
        };
        bails: {
            type: BooleanConstructor;
            default: () => boolean;
        };
        disabled: {
            type: BooleanConstructor;
            default: boolean;
        };
    };
} & ThisType<ComponentPublicInstance<Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, import("vue").ComponentOptionsBase<Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string>>>;

declare const Form: (new () => ComponentPublicInstance<{} & {
    as?: string;
    validationSchema?: Record<string, any>;
    initialValues?: Record<string, any>;
}, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, import("vue").VNodeProps & import("vue").AllowedComponentProps & import("vue").ComponentCustomProps, import("vue").ComponentOptionsBase<{} & {
    as?: string;
    validationSchema?: Record<string, any>;
    initialValues?: Record<string, any>;
}, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string>>) & ComponentOptionsBase<Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string> & {
    props: {
        as: {
            type: StringConstructor;
            default: string;
        };
        validationSchema: {
            type: ObjectConstructor;
            default: any;
        };
        initialValues: {
            type: ObjectConstructor;
            default: any;
        };
    };
} & ThisType<ComponentPublicInstance<Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, import("vue").ComponentOptionsBase<Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string>>>;

declare const ErrorMessage: (new () => ComponentPublicInstance<{
    name: string;
} & {
    as?: any;
}, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, import("vue").VNodeProps & import("vue").AllowedComponentProps & import("vue").ComponentCustomProps, import("vue").ComponentOptionsBase<{
    name: string;
} & {
    as?: any;
}, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string>>) & ComponentOptionsBase<Readonly<{
    name: string;
    as: any;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string> & {
    props: {
        as: {
            type: StringConstructor;
            default: any;
        };
        name: {
            type: StringConstructor;
            required: true;
        };
    };
} & ThisType<ComponentPublicInstance<Readonly<{
    name: string;
    as: any;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, Readonly<{
    name: string;
    as: any;
} & {}>, import("vue").ComponentOptionsBase<Readonly<{
    name: string;
    as: any;
} & {}>, () => import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}> | import("vue").VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, import("vue").ComponentOptionsMixin, import("vue").ComponentOptionsMixin, Record<string, any>, string>>>;

interface FieldOptions {
    value: Ref<any>;
    disabled: MaybeReactive<boolean>;
    immediate?: boolean;
    bails?: boolean;
    form?: FormController;
}
declare type RuleExpression = MaybeReactive<string | Record<string, any> | GenericValidateFunction>;
/**
 * Creates a field composite.
 */
declare function useField(fieldName: MaybeReactive<string>, rules: RuleExpression, opts?: Partial<FieldOptions>): {
    name: MaybeReactive<string>;
    value: Ref<any>;
    meta: {
        passed: ComputedRef<boolean>;
        failed: ComputedRef<boolean>;
        untouched: Ref<boolean>;
        touched: Ref<boolean>;
        dirty: Ref<boolean>;
        pristine: Ref<boolean>;
        valid: Ref<boolean>;
        invalid: Ref<boolean>;
        validated: Ref<boolean>;
        pending: Ref<boolean>;
        changed: Ref<boolean>;
    };
    errors: Ref<string[]>;
    errorMessage: ComputedRef<string>;
    aria: ComputedRef<{
        'aria-invalid': string;
        'aria-describedBy': string;
    }>;
    reset: () => void;
    validate: () => Promise<ValidationResult>;
    handleChange: (e: Event) => void;
    onBlur: () => void;
    disabled: MaybeReactive<boolean>;
    setValidationState: (result: ValidationResult) => ValidationResult;
};

interface FormOptions {
    validationSchema?: Record<string, GenericValidateFunction | string | Record<string, any>>;
    initialValues?: Record<string, any>;
}
declare function useForm(opts?: FormOptions): {
    errors: ComputedRef<any>;
    meta: Record<"failed" | "invalid" | "pending" | "untouched" | "touched" | "dirty" | "pristine" | "valid" | "passed" | "validated" | "changed", Ref<boolean>>;
    form: FormController;
    values: ComputedRef<any>;
    validate: () => Promise<boolean>;
    isSubmitting: Ref<boolean>;
    handleReset: () => void;
    handleSubmit: (fn?: SubmissionHandler) => (e: unknown) => Promise<void>;
    submitForm: (e: unknown) => Promise<void>;
};

export { ErrorMessage, Field, Form, configure, defineRule, useField, useForm, validate };
